/*
 * Copyright (c) 1993-2005 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS.
 */

/**
 ** gyroRqstsTaskCodels.c
 **
 ** Codels called by the execution task gyroRqstsTask
 **
 ** Author: 
 ** Date: 
 **
 **/

#ifdef VXWORKS
# include <vxWorks.h>
#else
# include <portLib.h>
#endif
#include "server/gyroHeader.h"

#include "h2mathLib.h"
#include "h2evnLib.h"
#include "h2timeLib.h"

#include "gyroConst.h"
#include "gyroLib.h"

#define	DEBUG	0

static BOOL gyroConnected=FALSE;

/*------------------------------------------------------------------------
 * Connect
 *
 * Description: 
 *
 * Bilans:      OK 
 *              S_gyro_CANNOT_CONNECT
 */

/* gyroConnectExec  -  codel EXEC de Connect
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT gyroConnectExec (GEN_STRING64* device, int *bilan)
{
  /* not connected yet */
  if (gyroConnected == GEN_FALSE) {
    if (!gyroInitAngle(device->name)) {
      *bilan = S_gyro_CANNOT_CONNECT;    
    }
    gyroConnected = TRUE;
    return ETHER;
  }
  /* already connected */
  *bilan = S_gyro_ALREADY_CONNECTED;
  return ETHER;
}


/*------------------------------------------------------------------------
 * Disconnect
 *
 * Description: 
 *
 * Bilans:      OK 
 *              S_gyro_CANNOT_DISCONNECT
 */

/* gyroDisconnectExec  -  codel EXEC de Disconnect
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT gyroDisconnectExec (int *bilan)
{
  gyroEnd();
  gyroConnected = FALSE;
  return ETHER;
}


/*------------------------------------------------------------------------
 * CalibrateOffset
 *
 * Description: 
 *
 * Bilans:      OK 
 *              S_gyro_FAILED
 */

/* gyroCalibrateOffsetExec  -  codel EXEC de CalibrateOffset
   Retourne:  EXEC EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT gyroCalibrateOffsetExec (int *bilan)
{
  if (!gyroUpdateWOffset()) {
    *bilan = S_gyro_FAILED;
  }
  return END;
}


/*------------------------------------------------------------------------
 * SetAngle
 *
 * Description: 
 *
 * Reports:      OK
 */

/* gyroSetAngleExec  -  codel EXEC of SetAngle
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
gyroSetAngleExec(double *angleInit, int *report)
{
  if (!gyroResetAngle(*angleInit)) {
    *report = S_gyro_FAILED;
  }
  return ETHER;

  SDI_F->angleInit = *angleInit;
}



/*------------------------------------------------------------------------
 *
 * gyroPermRead  --  codel of permanent activity
 *
 * Description: 
 * 
 * Reports:      OK
 * 
 * Returns:    OK or ERROR
 */

STATUS
gyroPermRead(int *report)
{    
  if (!gyroConnected)
    return OK;

  gyroReadCorrAngle(&SDI_F->angle.yaw);
#if DEBUG>=3
  printf ("angle %g\n", SDI_F->angle.yaw);
#endif

  return OK;
}



/*------------------------------------------------------------------------
 * Log
 *
 * Description: 
 *
 * Reports:      OK
 *              S_gyro_CANNOT_OPEN_FILE
 *              S_gyro_CANNOT_WRITE_FILE
 */

static FILE *gyroLogFile;

/* gyroLogStart  -  codel START of Log
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
gyroLogStart(GEN_STRING128 *fileName, int *report)
{
  if (!(gyroLogFile = fopen(fileName->name, "a"))) {
    printf ("gyro: cannot open odometer file %s\n", fileName->name);
    *report = S_gyro_CANNOT_OPEN_FILE;
    return ETHER;
  }

  printf ("gyroLogStart: log file %s found\n", fileName->name);
  return EXEC;
}

/* gyroLogExec  -  codel EXEC of Log
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
gyroLogExec(GEN_STRING128 *fileName, int *report)
{
  H2TIMESPEC tv;

  /* get date */
  h2GetTimeSpec(&tv);

  /* save date */
  if (!fprintf(gyroLogFile, "%lf\n", 
	       tv.tv_sec + tv.tv_nsec*1.0E-9)) {
    printf ("gyro: cannot log in %s\n", fileName->name);
    gyroLogFile = 0;
    *report = S_gyro_CANNOT_WRITE_FILE;
    return ETHER;
  }

  /* save POM_ME_POS */
  fprintf(gyroLogFile, "angleCorr = %g\n", SDI_F->angle.yaw);

  return EXEC;
}

/* gyroLogInter  -  codel END of Log
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
gyroLogInter(GEN_STRING128 *fileName, int *report)
{

  fclose(gyroLogFile);
  gyroLogFile = NULL;

  return ETHER;
}

